#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributers.

from fenrirscreenreader.core import debug
import re
import time
import threading

class command():
    def __init__(self):
        pass
        
    def initialize(self, environment):
        self.env = environment
        # Use commandBuffer like other commands
        if 'progressMonitoring' not in self.env['commandBuffer']:
            # Check if progress monitoring should be enabled by default from settings
            try:
                defaultEnabled = self.env['runtime']['settingsManager'].getSettingAsBool('sound', 'progressMonitoring')
            except:
                # If setting doesn't exist, default to False
                defaultEnabled = False
            self.env['commandBuffer']['progressMonitoring'] = defaultEnabled
            self.env['commandBuffer']['lastProgressTime'] = 0
            self.env['commandBuffer']['lastProgressValue'] = -1
        
    def shutdown(self):
        self.stopProgressMonitoring()
        
    def getDescription(self):
        return _('Toggle progress bar monitoring with ascending tones')
        
    def run(self):
        # Check if commandBuffer exists
        if 'progressMonitoring' not in self.env['commandBuffer']:
            self.env['commandBuffer']['progressMonitoring'] = False
            self.env['commandBuffer']['lastProgressTime'] = 0
            self.env['commandBuffer']['lastProgressValue'] = -1
        
        if self.env['commandBuffer']['progressMonitoring']:
            self.stopProgressMonitoring()
            self.env['runtime']['outputManager'].presentText(_("Progress monitoring disabled"), interrupt=True)
        else:
            self.startProgressMonitoring()
            self.env['runtime']['outputManager'].presentText(_("Progress monitoring enabled"), interrupt=True)
            
    def startProgressMonitoring(self):
        self.env['commandBuffer']['progressMonitoring'] = True
        self.env['commandBuffer']['lastProgressTime'] = time.time()
        self.env['commandBuffer']['lastProgressValue'] = -1
        # Don't control speech - let user decide with silence_until_prompt
        
    def stopProgressMonitoring(self):
        self.env['commandBuffer']['progressMonitoring'] = False
        # Don't control speech - progress monitor is beep-only
            
    def detectProgress(self, text):
        if not self.env['runtime']['progressMonitoring']:
            return
            
        currentTime = time.time()
        
        # Pattern 1: Percentage (50%, 25.5%, etc.)
        percentMatch = re.search(r'(\d+(?:\.\d+)?)\s*%', text)
        if percentMatch:
            percentage = float(percentMatch.group(1))
            if percentage != self.env['runtime']['lastProgressValue']:
                self.playProgressTone(percentage)
                self.env['runtime']['lastProgressValue'] = percentage
                self.env['runtime']['lastProgressTime'] = currentTime
            return
            
        # Pattern 2: Fraction (15/100, 3 of 10, etc.)
        fractionMatch = re.search(r'(\d+)\s*(?:of|/)\s*(\d+)', text)
        if fractionMatch:
            current = int(fractionMatch.group(1))
            total = int(fractionMatch.group(2))
            if total > 0:
                percentage = (current / total) * 100
                if percentage != self.env['runtime']['lastProgressValue']:
                    self.playProgressTone(percentage)
                    self.env['runtime']['lastProgressValue'] = percentage
                    self.env['runtime']['lastProgressTime'] = currentTime
                return
                
        # Pattern 3: Progress bars ([####    ], [====>   ], etc.)
        barMatch = re.search(r'\[([#=\-\*]+)([^\]]*)\]', text)
        if barMatch:
            filled = len(barMatch.group(1))
            total = filled + len(barMatch.group(2))
            if total > 0:
                percentage = (filled / total) * 100
                if percentage != self.env['runtime']['lastProgressValue']:
                    self.playProgressTone(percentage)
                    self.env['runtime']['lastProgressValue'] = percentage
                    self.env['runtime']['lastProgressTime'] = currentTime
                return
                
        # Pattern 4: Generic activity indicators (Loading..., Working..., etc.)
        activityPattern = re.search(r'(loading|processing|working|installing|downloading|compiling|building).*\.{2,}', text, re.IGNORECASE)
        if activityPattern:
            # Play a steady beep every 2 seconds for ongoing activity
            if currentTime - self.env['runtime']['lastProgressTime'] >= 2.0:
                self.playActivityBeep()
                self.env['runtime']['lastProgressTime'] = currentTime
                
    def playProgressTone(self, percentage):
        # Map 0-100% to 400-1200Hz frequency range
        frequency = 400 + (percentage * 8)
        frequency = max(400, min(1200, frequency))  # Clamp to safe range
        self.env['runtime']['outputManager'].playFrequence(frequency, 0.15, interrupt=False)
        
    def playActivityBeep(self):
        # Single tone for generic activity
        self.env['runtime']['outputManager'].playFrequence(800, 0.1, interrupt=False)
        
    def setCallback(self, callback):
        pass