#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Fenrir TTY screen reader
# By Chrys, Storm Dragon, and contributers.

from fenrirscreenreader.core import debug

class command():
    def __init__(self):
        pass
        
    def initialize(self, environment):
        self.env = environment
        
    def shutdown(self):
        pass
        
    def getDescription(self):
        return 'Detects progress patterns for progress bar monitoring'
        
    def run(self):
        # Only run if progress monitoring is enabled
        try:
            if 'progressMonitoring' in self.env['commandBuffer'] and self.env['commandBuffer']['progressMonitoring']:
                # Only check new incoming text (newDelta), but filter out screen changes
                if self.env['screen']['newDelta'] and self.isRealProgressUpdate():
                    self.detectProgress(self.env['screen']['newDelta'])
        except Exception as e:
            # Silently ignore errors to avoid disrupting normal operation
            pass
            
    def isRealProgressUpdate(self):
        """Check if this is a real progress update vs screen change/window switch"""
        # If the screen/application changed, it's not a progress update
        if self.env['runtime']['screenManager'].isScreenChange():
            return False
            
        # If there was a large cursor movement, it's likely navigation, not progress
        if self.env['runtime']['cursorManager'].isCursorVerticalMove():
            xMove = abs(self.env['screen']['newCursor']['x'] - self.env['screen']['oldCursor']['x'])
            yMove = abs(self.env['screen']['newCursor']['y'] - self.env['screen']['oldCursor']['y'])
            # Large movements suggest navigation, not progress output
            if yMove > 2 or xMove > 20:
                return False
                
        # Check if delta is too large (screen change) vs small incremental updates
        deltaLength = len(self.env['screen']['newDelta'])
        if deltaLength > 200:  # Allow longer progress lines like Claude Code's status
            return False
            
        return True
            
    def detectProgress(self, text):
        import re
        import time
        
        currentTime = time.time()
        
        # Debug: Print what we're checking
        self.env['runtime']['debug'].writeDebugOut("Progress detector checking: '" + text + "'", debug.debugLevel.INFO)
        
        # Check if progress monitoring should automatically stop
        self.checkProgressCompletion(text, currentTime)
        
        # Pattern 1: Percentage (50%, 25.5%, etc.)
        percentMatch = re.search(r'(\d+(?:\.\d+)?)\s*%', text)
        if percentMatch:
            percentage = float(percentMatch.group(1))
            self.env['runtime']['debug'].writeDebugOut("Found percentage: " + str(percentage), debug.debugLevel.INFO)
            if percentage != self.env['commandBuffer']['lastProgressValue']:
                self.env['runtime']['debug'].writeDebugOut("Playing tone for: " + str(percentage), debug.debugLevel.INFO)
                self.playProgressTone(percentage)
                self.env['commandBuffer']['lastProgressValue'] = percentage
                self.env['commandBuffer']['lastProgressTime'] = currentTime
            return
            
        # Pattern 1b: Time/token activity (not percentage-based, so use single beep)
        timeMatch = re.search(r'(\d+)s\s', text)
        tokenMatch = re.search(r'(\d+)\s+tokens', text)
        
        if timeMatch or tokenMatch:
            # For non-percentage progress, use a single activity beep every 2 seconds
            if currentTime - self.env['commandBuffer']['lastProgressTime'] >= 2.0:
                self.env['runtime']['debug'].writeDebugOut("Playing activity beep for Claude Code progress", debug.debugLevel.INFO)
                self.playActivityBeep()
                self.env['commandBuffer']['lastProgressTime'] = currentTime
            return
            
        # Pattern 2: Fraction (15/100, 3 of 10, etc.)
        fractionMatch = re.search(r'(\d+)\s*(?:of|/)\s*(\d+)', text)
        if fractionMatch:
            current = int(fractionMatch.group(1))
            total = int(fractionMatch.group(2))
            if total > 0:
                percentage = (current / total) * 100
                if percentage != self.env['commandBuffer']['lastProgressValue']:
                    self.playProgressTone(percentage)
                    self.env['commandBuffer']['lastProgressValue'] = percentage
                    self.env['commandBuffer']['lastProgressTime'] = currentTime
                return
                
        # Pattern 3: Progress bars ([####    ], [====>   ], etc.)
        barMatch = re.search(r'\[([#=\-\*]+)([^\]]*)\]', text)
        if barMatch:
            filled = len(barMatch.group(1))
            total = filled + len(barMatch.group(2))
            if total > 0:
                percentage = (filled / total) * 100
                if percentage != self.env['commandBuffer']['lastProgressValue']:
                    self.playProgressTone(percentage)
                    self.env['commandBuffer']['lastProgressValue'] = percentage
                    self.env['commandBuffer']['lastProgressTime'] = currentTime
                return
                
        # Pattern 4: Generic activity indicators (Loading..., Working..., etc.)
        activityPattern = re.search(r'(loading|processing|working|installing|downloading|compiling|building).*\.{2,}', text, re.IGNORECASE)
        if activityPattern:
            # Play a steady beep every 2 seconds for ongoing activity
            if currentTime - self.env['commandBuffer']['lastProgressTime'] >= 2.0:
                self.playActivityBeep()
                self.env['commandBuffer']['lastProgressTime'] = currentTime
                
    def playProgressTone(self, percentage):
        # Map 0-100% to 400-1200Hz frequency range  
        frequency = 400 + (percentage * 8)
        frequency = max(400, min(1200, frequency))  # Clamp to safe range
        
        # Use Sox directly for clean quiet tones like: play -qn synth .1 tri 400 gain -8
        self.playQuietTone(frequency, 0.1)
        
    def playActivityBeep(self):
        # Single tone for generic activity
        self.playQuietTone(800, 0.08)
        
    def playQuietTone(self, frequency, duration):
        """Play a quiet tone using Sox directly"""
        import subprocess
        import shlex
        
        # Build the Sox command: play -qn synth <duration> tri <frequency> gain -8
        command = f"play -qn synth {duration} tri {frequency} gain -8"
        
        try:
            # Only play if sound is enabled
            if self.env['runtime']['settingsManager'].getSettingAsBool('sound', 'enabled'):
                subprocess.Popen(shlex.split(command), stdin=None, stdout=None, stderr=None, shell=False)
        except Exception as e:
            self.env['runtime']['debug'].writeDebugOut("Sox tone error: " + str(e), debug.debugLevel.ERROR)
        
    def checkProgressCompletion(self, text, currentTime):
        """Check if progress is complete and should auto-disable monitoring"""
        # Progress monitor is now beep-only - user controls speech separately
        # Only auto-disable on clear 100% completion for convenience  
        import re
        
        if re.search(r'100\s*%', text):
            self.env['runtime']['debug'].writeDebugOut("Progress complete: 100%", debug.debugLevel.INFO)
            self.stopProgressMonitoring()
            return
                    
    def stopProgressMonitoring(self):
        """Stop progress monitoring - beep-only, no speech control"""
        self.env['commandBuffer']['progressMonitoring'] = False
        # Just disable monitoring, don't touch speech settings
            
    def setCallback(self, callback):
        pass